﻿namespace Falcon.SugarApi.Proxy
{
    /// <summary>
    /// 对象代理接口
    /// </summary>
    public abstract class ProxyBase
    {
        /// <summary>
        /// 对象代理接口
        /// </summary>
        /// <param name="target">代理对象</param>
        public ProxyBase(object target) {
            Target = target;
        }

        /// <summary>
        /// 代理对象
        /// </summary>
        public virtual object Target { get; }
        /// <summary>
        /// 获取对象属性的代理
        /// </summary>
        /// <param name="prop">属性名称</param>
        /// <returns></returns>
        /// <exception cref="PropNotFoundException">没有找到属性</exception>
        public virtual object? GetFunc(string prop) {
            var p = this.Target.GetType().GetProperty(prop) ?? throw new PropNotFoundException(prop);
            return p.GetValue(this.Target);
        }
        /// <summary>
        /// 设置对象属性的代理
        /// </summary>
        /// <param name="prop">属性名称</param>
        /// <param name="value">属性值</param>
        /// <exception cref="PropNotFoundException">没有找到属性</exception>
        public virtual void SetAction(string prop,object? value) {
            var p = this.Target.GetType().GetProperty(prop) ?? throw new PropNotFoundException(prop);
            p.SetValue(this.Target,value);
        }
        /// <summary>
        /// 执行代理对象方法
        /// </summary>
        /// <param name="name">方法名称</param>
        /// <param name="args">传入的参数</param>
        /// <returns>方法返回值</returns>
        /// <exception cref="MethodNotFoundException">没有找到方法</exception>
        public virtual object? Invoke(string name,params object[] args) {
            var m = this.Target.GetType().GetMethod(name) ?? throw new MethodNotFoundException(name);
            return m.Invoke(this.Target,args);
        }

    }
}